###### hosts.tcl V1.0.0 - ppslim @ efnet / egghelpformum.com
#    #
#  Designed to help manage user hostmasks with ease
#  Currently displays host statistics, check nickname / handle to see if he/she has a matching host,
#  If they are online and it displays a message saying it doesn;t recognise them, you know to add them.
#  Lists hostsmask of a user in numbered order, for use with the removal command.
#  You can remove hostmasks using numbers, using the format of the notes removal command.
#  EG: .hosts remove testnick 1,3,5-7,9 - thus removing hosts numbered 1 3 5 6 7 9.
#    #
#  VERSIONS:
#  V1.0.0 - First version, RFC ppslim@ntlworld.com
#    #
#  TODO:
#  Don't know yet, e-mail me with ideas @ ppslim@ntlworld.com or catch me
#  on the egghelp.org forum (www.egghelpforum.com)
#    #
######

bind dcc n|n hosts dcc:hosts
proc dcc:hosts {hand idx arg} {
  set op [lindex $arg 0]
  switch $op {
    stats {
      set h 0
      set bi 0
      set bn ""
      set p [userlist]
      foreach i $p {
        set f [llength [getuser $i hosts]]
        incr h $f
        if {$f >= $bi} {
          lappend bn $i
          if {$f > $bi} {
            set bi $f
            set bn $i
          }
        }
      }
      set u [llength $p]
      putdcc $idx "There are currently $h hosts and $u users"
      putdcc $idx "That makes about [expr $h / "${u}.0"] hosts per user"
      putdcc $idx "The users with the most hosts:" 
      putdcc $idx "$bn has $bi"
    }
    list {
      set u [lindex $arg 1]
      if {[validuser $u]} {
        putdcc $idx "Hosts list for $u"
        set c 0
        set h [getuser $u hosts]
        set n [llength $h]
        while {$c != $n} {
          incr c
          putdcc $idx "  ${c}: [lindex $h [expr $c - 1]]"
        }
        putdcc $idx "Listed $n hosts for $u"
      } else {
        pudcc $idx "$u is not a valid handle"
      }
    }
    check {
      set u [lindex $arg 1]
      set f [hand2nick $u]
      if {$f != ""} {
        putdcc $idx "Found $f - He matches a host for $u"
      } else {
        putdcc $idx "No nickname matching hosts for $u, trying a nickname match"
        set f [nick2hand $u]
        if {(($f != "") && ($f != "*"))} {
          putdcc $idx "Found $u on a channel. Host matches. $u = $f"
        } elseif {$f == "*"} {
          putdcc $idx "Found $u on a channel. But there are no matching hosts"
        } else {
          putdcc $idx "Could not find a nickname matching ${u}. They must be offline"
        }
      }
    }
    remove {
      set u [lindex $arg 1]
      set args [lindex $arg 2]
      if {![validuser $u]} {
        putdcc $idx "$u is not a valid handle"
      } else {
        set h [getuser $u hosts]
        if {$args == "all"} {
          set w [llength $h]
          setuser $u hosts
          putdcc $idx "Cleared $w (all) hosts for $u"
        } else {
          set int [listint $args]
          set m 1
          set n ""
          set r 0
          set k 0
          setuser $u hosts
          foreach i $h {
            if {[lsearch -exact $int $m] < 0} {
              setuser $u hosts $i
              incr k
            }
            incr m
          }
          putdcc $idx "Removed [expr [llength $h] - $k] hosts from $u"
        }
      }
    }
    default {
      putdcc $idx "Help for host managment"
      putdcc $idx "  stats               - Get information about the hosts in the usefile"
      putdcc $idx "  list <handle>       - Get a list of hosts for a handle"
      putdcc $idx "  check <handle/nick> - Check if there is a matching host"
      putdcc $idx "  remove <hand> list> - Remove a large amount of hosts, using the notes style list"
    }
  }
}




proc listint {list} {
  set r ""
  set l [llength [split $list ,]]
  set h 0
  while {$h != $l} {
    set t [lindex [split $list ,] $h]
    if {[llength [split $t -]] > 1} {
      set bl [lindex [split $t -] 0]
      set bh [lindex [split $t -] 1]
      for {set x $bl} {$x <= $bh} {incr x} {
        lappend r $x
      }
    } else {
      lappend r $t
    }
    incr h
  }
  return $r
}

